
from lightcast_smart_dataset.insight.occupation_insight import OccupationInsight, \
    BasicOccupationInsightResponseParser, \
    OccupationInsightRequestClient, \
    OccupationInsightRequest, \
    OccupationInsightResponse


class UK:

    def __init__(self, username: str, password: str) -> None:
        super(UK, self).__init__()
        url = "https://solutions-api.lightcast.io/smart-dataset/occupation-insight/v1/uk"
        self.__occupation_insight = OccupationInsight(url=url,
                                                      username=username,
                                                      password=password,
                                                      insight_response=BasicOccupationInsightResponseParser(),
                                                      insight_request=OccupationInsightRequestClient())

    def getSocOccupationInsight(self,
                                occupation: str = "",
                                occupation_level: str = "4",
                                area: str = "",
                                area_level: str = "3") -> OccupationInsightResponse:
        request = OccupationInsightRequest()
        request.occupation = occupation
        request.area = area
        request.occupation_level = occupation_level
        request.area_level = area_level
        request.occupation_classification = "soc"
        request.area_classification = "nuts"

        response = self.__occupation_insight.insight(request)
        return response

    def getOccupationInsight(self,
                             occupation: str = "",
                             occupation_level: str = "4",
                             area: str = "",
                             area_level: str = "3") -> OccupationInsightResponse:
        request = OccupationInsightRequest()
        request.occupation = occupation
        request.area = area
        request.occupation_level = occupation_level
        request.area_level = area_level
        request.occupation_classification = "occupation"
        request.area_classification = "nuts"

        response = self.__occupation_insight.insight(request)
        return response



import requests
import json


class OAuthClient:

    def __init__(self, auth_url, username, password):
        self.__auth_url = auth_url
        self.__username = username
        self.__password = password

    def getAuthorizationString(self):
        login_payload = {
            "username": self.__username,
            "password": self.__password
        }

        headers = {'Content-Type': "application/json"}

        response = requests.request("POST", self.__auth_url,
                                    data=json.dumps(login_payload),
                                    headers=headers)
        response = json.loads(response.text)
        tokens = response["tokens"]
        access_token = tokens["access_token"]

        return f'Bearer {access_token}'



# coding=utf-8
import setuptools

setuptools.setup(
    name='lightcast-smart-dataset',
    version='0.1.29',
    scripts=['lightcast-smart-dataset'],
    author="Global Data Science Team - Lightcast",
    author_email="globaldatascience@lightcast.io",
    description="Utility use the Lightcast Global Smart Dataset",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Lightcast-Global-Innovation/global-smart-dataset",
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.7',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['requests',
                      'xlsxwriter',
                      'pandas',
                      'docopt']
)




from lightcast_smart_dataset. insight.taxonomy_insight import TaxonomyInsight, \
    TaxonomyRequestClient, \
    TaxonomyRequest, \
    TaxonomyResponse, \
    BasicTaxonomyResponseParser


class Taxonomy:

    def __init__(self, username: str, password: str) -> None:
        super(Taxonomy, self).__init__()
        self.__client = TaxonomyInsight(username=username,
                                        password=password,
                                        response=BasicTaxonomyResponseParser(),
                                        request=TaxonomyRequestClient())

    def getSocLevel4(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "uk"
        request.facet = "soc4"

        response = self.__client.get(request)
        return response.codelist['data']

    def getSocLevel3(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "uk"
        request.facet = "soc3"

        response = self.__client.get(request)
        return response.codelist['data']

    def getSocLevel2(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "uk"
        request.facet = "soc2"

        response = self.__client.get(request)
        return response.codelist['data']

    def getSocLevel1(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "uk"
        request.facet = "soc1"

        response = self.__client.get(request)
        return response.codelist['data']

    def getOccupation(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "uk"
        request.facet = "occupation"

        response = self.__client.get(request)
        return response.codelist['data']

    def getUkNuts3(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "uk"
        request.facet = "nuts3"

        response = self.__client.get(request)
        return response.codelist['data']

    def getUkLocalAreaUnit(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "uk"
        request.facet = "nuts4"

        response = self.__client.get(request)
        return response.codelist['data']

    def getGlobalMarket(self) -> TaxonomyResponse:
        request = TaxonomyRequest()
        request.source = "global"
        request.facet = "market"

        response = self.__client.get(request)
        return response.codelist['data']




from lightcast_smart_dataset.lightcast_client.uk_dataset import UK
from lightcast_smart_dataset.lightcast_client.global_dataset import Global
from lightcast_smart_dataset.lightcast_client.taxonomy import Taxonomy


class LightcastSmartDataset():

    def __init__(self, username: str, password: str) -> None:
        super(LightcastSmartDataset, self).__init__()
        self.__setUsername(username)
        self.__setPassword(password)

    def __setUsername(self, username: str) -> None:
        if not isinstance(username, str):
            raise TypeError("username is not an instance of str.")

        self.__username = username

    def __setPassword(self, password: str) -> None:
        if not isinstance(password, str):
            raise TypeError("password is not an instance of str.")

        self.__password = password

    def ukDataset(self) -> UK:
        return UK(self.__username, self.__password)

    def globalDataset(self) -> Global:
        return Global(self.__username, self.__password)

    def taxonomy(self) -> Taxonomy:
        return Taxonomy(self.__username, self.__password)



from abc import ABCMeta
from lightcast_smart_dataset.lightcast_client.OAuthClient import OAuthClient
import requests
import json


class TaxonomyRequest:

    def __init__(self):
        self.source = None
        self.facet = None

    @property
    def source(self) -> str:
        return self.__source

    @source.setter
    def source(self, source: str) -> None:
        self.__source = source

    @property
    def facet(self) -> str:
        return self.__facet

    @facet.setter
    def facet(self, facet: str) -> None:
        self.__facet = facet


class TaxonomyResponse:

    def __init__(self):
        pass

    @property
    def raw_response(self) -> str:
        return self.__raw_response

    @raw_response.setter
    def raw_response(self, raw_response: str) -> None:
        self.__raw_response = raw_response

    @property
    def codelist(self) -> list:
        return self.__codelist

    @codelist.setter
    def codelist(self, codelist: list) -> None:

        self.__codelist = codelist


class ResponseTaxonomy(metaclass=ABCMeta):

    def deserialize(response) -> TaxonomyResponse:
        raise NotImplementedError()


class RequestTaxonomy(metaclass=ABCMeta):

    def write_request(self, url: str, request: TaxonomyRequest, oauth: OAuthClient) -> dict:
        raise NotImplementedError()


class TaxonomyRequestClient(RequestTaxonomy):

    def __create_parse_command(self,
                               url: str,
                               request: TaxonomyRequest) -> str:

        return url + "/" + request.source + "/" + request.facet + "?limit=10000"

    def write_request(self, url: str, request: TaxonomyRequest, oauth: OAuthClient) -> str:
        headers = {'Content-Type': "application/json",
                   'Accept': "application/json",
                   'Authorization': oauth.getAuthorizationString()}
        final_url = self.__create_parse_command(url, request)
        response = requests.get(final_url, headers=headers)
        return response.text.encode('utf8')


class TaxonomyInsight:

    def __init__(self,
                 url: str = "https://solutions-api.lightcast.io/smart-dataset/taxonomies",
                 auth_url: str = "https://solutions-api.lightcast.io/api/users/login",
                 username: str = "",
                 password: str = "",
                 response: ResponseTaxonomy = None,
                 request: RequestTaxonomy = None) -> None:
        self.__url = url
        self.__auth_url = auth_url
        self.__username = username
        self.__password = password
        self.__setResponse(response)
        self.__setRequest(request)

    def __setRequest(self, request: RequestTaxonomy) -> None:
        if not isinstance(request, RequestTaxonomy):
            raise TypeError("request is not an instance of RequestTaxonomy.")

        self.__request = request

    def __setResponse(self, response: ResponseTaxonomy) -> None:
        if not isinstance(response, ResponseTaxonomy):
            raise TypeError("response is not an instance of ResponseTaxonomy.")

        self.__response = response

    def get(self, request: TaxonomyRequest) -> TaxonomyResponse:
        try:
            return self.__get(request)
        except Exception as e:
            raise e

    def __get(self, request: TaxonomyRequest) -> TaxonomyResponse:
        oauth = OAuthClient(self.__auth_url, self.__username, self.__password)
        raw_response = self.__request.write_request(self.__url, request, oauth)
        response = self.__response.deserialize(raw_response)
        return response


class BasicTaxonomyResponseParser(ResponseTaxonomy):

    def __init__(self):
        pass

    def deserialize(self, raw_response: str):

        print(raw_response)
        json_response = json.loads(raw_response)
        response = TaxonomyResponse()
        response.raw_response = raw_response
        response.codelist = json_response

        return response
